"""M4 skills package for AI coding tools."""

from m4.skills.installer import (
    AI_TOOLS,
    AITool,
    get_all_installed_skills,
    get_available_tools,
    get_installed_skills,
    get_skills_source,
    install_skills,
)

__all__ = [
    "AI_TOOLS",
    "AITool",
    "get_all_installed_skills",
    "get_available_tools",
    "get_installed_skills",
    "get_skills_source",
    "install_skills",
]
